using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;

public class AgentController : MonoBehaviour
{
    GameObject director;
    TSPsolver tspsolver;
    //TSPsolver2 tspsolver;
    //TSPsolver3 tspsolver;
    private float CurrentTime { get; set; }
    [Header("Settings"), SerializeField] private float updateSpan = 0.1f;
    private float UpdateSpan => updateSpan;

    void Start()
    {
        this.director = GameObject.Find("TSPsolver");
        this.tspsolver = this.director.GetComponent<TSPsolver>();
        CurrentTime = 0.0f;
    }

    void FixedUpdate()
    {
        CurrentTime += Time.deltaTime;
        if(CurrentTime > UpdateSpan) {
            MoveAgent();
            CurrentTime = 0.0f;
        }
    }

    void MoveAgent()
    {
        // agentを一周させる
        int CurrentCity = 0;
        float MyDistance = 0;
        List<List<float>> problist = tspsolver.ProbList;
        int CityCount = problist.Count;
        List<int> VisitedList = new List<int>();

        List<bool> isVisited = new List<bool>();
        for(int i = 0;i < CityCount;i++) isVisited.Add(false);

        VisitedList.Add(0);
        isVisited[0] = true;


        // 一周するまでのループ
        while(VisitedList.Count != CityCount)
        {
            // 未到達の都市についてproblistの値の和を取る(移動確率を求めるための分母)
            float probsum = 0;
            for(int i = 0;i < CityCount;i++)
            {
                if(isVisited[i] == false) probsum += problist[CurrentCity][i];
            }

            // 確率的に次に移動する年を決定する
            int NextCity = -1;
            float prob = 0.0f;
            float r = Random.Range(0.0f, 1.0f);
            for(int i = 0;i < CityCount;i++)
            {
                if(isVisited[i] == false)
                {
                    prob += problist[CurrentCity][i];
                    Debug.Log(prob);
                    if(r < prob/probsum)
                    {
                        NextCity = i;
                        break;
                    }
                }
            }

            tspsolver.RenewPheromoneList(CurrentCity, NextCity);
            MyDistance += this.tspsolver.DistanceList[CurrentCity][NextCity];
            VisitedList.Add(NextCity);
            isVisited[NextCity] = true;

            CurrentCity = NextCity;
        }
        tspsolver.RenewPheromoneList(CurrentCity, 0);
        MyDistance += this.tspsolver.DistanceList[CurrentCity][0];

        // 最短距離を更新していたらtspsolverに反映
        if(MyDistance < this.tspsolver.Shortest)
        {
            // Debug.Log("update shortest");
            this.tspsolver.Shortest = MyDistance;
            this.tspsolver.RenewShortestPath(VisitedList);
        }
    }
}
